/***************************************************************************
 *   Copyright (C) 2016 by Pere Ràfols Soler                               *
 *   sapista2@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef PLUGINS_URIS_H
#define PLUGINS_URIS_H

//Plugin uris
#define EQ1QM_URI "http://eq10q.sourceforge.net/eq/eq1qm"
#define EQ1QS_URI "http://eq10q.sourceforge.net/eq/eq1qs"
#define EQ4QM_URI "http://eq10q.sourceforge.net/eq/eq4qm"
#define EQ4QS_URI "http://eq10q.sourceforge.net/eq/eq4qs"
#define EQ6QM_URI "http://eq10q.sourceforge.net/eq/eq6qm"
#define EQ6QS_URI "http://eq10q.sourceforge.net/eq/eq6qs"
#define EQ10QM_URI "http://eq10q.sourceforge.net/eq/eq10qm"
#define EQ10QS_URI "http://eq10q.sourceforge.net/eq/eq10qs"
#define GATE_MONO_URI "http://eq10q.sourceforge.net/gate"
#define GATE_STEREO_URI "http://eq10q.sourceforge.net/gate_stereo"
#define COMPRESSOR_MONO_URI "http://eq10q.sourceforge.net/compressor"
#define COMPRESSOR_SC_MONO_URI "http://eq10q.sourceforge.net/compressor_sc"
#define COMPRESSOR_STEREO_URI "http://eq10q.sourceforge.net/compressor_stereo"
#define COMPRESSOR_SC_STEREO_URI "http://eq10q.sourceforge.net/compressor_stereo_sc"
#define LR2MS_URI "http://eq10q.sourceforge.net/matrix_lr2ms"
#define MS2LR_URI "http://eq10q.sourceforge.net/matrix_ms2lr"

//Plugins GUI uris
#define EQ10Q_GUI_URI "http://eq10q.sourceforge.net/eq/eq10q/gui"
#define DYNAMICS_GUI_URI "http://eq10q.sourceforge.net/dynamics/gui"
#define MIDSIDEMAT_GUI_URI "http://eq10q.sourceforge.net/matrix_ms/gui"

#endif
